"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var path = require("path");
var webpack = require("webpack");
var HtmlWebpackPlugin = require("html-webpack-plugin");

var CSSLoader = "css-loader?sourceMap&-minimize&modules&importLoaders=1&localIdentName=[name]__[local]__[hash:base64:5]";

module.exports = function (extraDefines) {
  return {
    devtool: "eval-source-map",
    mode: "development",
    resolve: {
      extensions: [".jsx", ".js"],
      symlinks: false,
      cacheWithContext: false,
      modules: ["node_modules"],
      alias: {
        app: path.resolve(__dirname, "app")
      }
    },
    entry: ["babel-polyfill", "webpack-dev-server/client?http://" + process.env.HOSTNAME + ":3000", "webpack/hot/only-dev-server", "react-hot-loader/patch", path.join(__dirname, "app/globals.js"), path.join(__dirname, "app/index.js")],
    output: {
      path: path.join(__dirname, "dist"),
      filename: "[name].js",
      chunkFilename: "[name].bundle.js",
      publicPath: "/"
    },
    plugins: [new HtmlWebpackPlugin({
      template: "./src/client/app/index.tpl.html",
      inject: "body",
      filename: "index.html"
    }), new webpack.HotModuleReplacementPlugin(), new webpack.NoEmitOnErrorsPlugin(), new webpack.DefinePlugin(_extends({
      API_URL: JSON.stringify(process.env.API_URL),
      TITLE: JSON.stringify(process.env.TITLE),
      __DEV__: true,
      __LIVE__: true,
      NODE_ENV: JSON.stringify(process.env.NODE_ENV)
    }, extraDefines))],
    devServer: {
      host: process.env.HOSTNAME,
      disableHostCheck: true
    },
    // recordsPath: path.resolve( __dirname, "./recordsPath.json" ),
    module: {
      rules: [{
        test: /\.(js|jsx)?$/,
        include: path.join(__dirname, "app"),
        use: ["babel-loader"]
      }, {
        test: /\.css$/,
        include: path.join(__dirname, "app"),
        loader: "style-loader!css-loader?modules&importLoaders=1&localIdentName=[name]__[local]___[hash:base64:5]!less-loader!postcss-loader"
      }, {
        test: /\.(jpe?g|png|gif)$/i,
        include: path.join(__dirname, "app"),
        loaders: ["file-loader?hash=sha512&digest=hex&name=[hash].[ext]", "image-webpack-loader"]
      }, {
        test: /\.svg$/i,
        include: path.join(__dirname, "app"),
        loaders: ["file-loader?hash=sha512&digest=hex&name=[hash].[ext]"]
      }, {
        test: /\.json$/,
        include: path.join(__dirname, "app"),
        loader: "json-loader"
      }, {
        test: /\.css$/,
        loaders: ["style-loader", CSSLoader],
        include: path.join(__dirname, "/node_modules/font-awesome")
      }, {
        test: /\.scss$/,
        loaders: ["style-loader", CSSLoader],
        include: path.join(__dirname, "/app/components/NotificationsTheme/styles/")
      }, {
        test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: "url-loader?limit=10000&minetype=application/font-woff",
        include: path.join(__dirname, "/node_modules/font-awesome")
      }, {
        test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: "file-loader",
        include: path.join(__dirname, "/node_modules/font-awesome")
      }]
    }
  };
};