"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

require("babel-polyfill");

require("dotenv/config");

var _express = require("express");

var _express2 = _interopRequireDefault(_express);

var _compression = require("compression");

var _compression2 = _interopRequireDefault(_compression);

var _expressGraphql = require("express-graphql");

var _expressGraphql2 = _interopRequireDefault(_expressGraphql);

var _graphql = require("./graphql");

var _https = require("https");

var _https2 = _interopRequireDefault(_https);

var _http = require("http");

var _http2 = _interopRequireDefault(_http);

var _mssql = require("mssql");

var _mssql2 = _interopRequireDefault(_mssql);

var _moment = require("moment");

var _moment2 = _interopRequireDefault(_moment);

var _io = require("./io");

var _io2 = _interopRequireDefault(_io);

var _CUCM = require("./db/helpers/CUCM/");

var _CUCM2 = _interopRequireDefault(_CUCM);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _cors = require("cors");

var _cors2 = _interopRequireDefault(_cors);

var _extensions = require("./extensions");

var _scheduledTasks = require("./scheduledTasks");

var _scheduledTasks2 = _interopRequireDefault(_scheduledTasks);

var _generateFlags = require("./jobs/generateFlags");

var _generateFlags2 = _interopRequireDefault(_generateFlags);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* globals PRODUCTION */


_moment2.default.locale("pt");

global.PRODUCTION = process.env.PRODUCTION;
global.MASTER_KEY = process.env.MASTER_KEY;
global.DYNAMICS = process.env.DYNAMICS;
global.HOSTNAME = process.env.HOSTNAME;
global.ERP = process.env.ERP;
global.UPLOAD_PATH = process.env.UPLOAD_PATH;
global.HOSTNAME = process.env.HOSTNAME;
global.GRAPHQL_PORT = process.env.GRAPHQL_PORT;

var privateKey = PRODUCTION ? _fs2.default.readFileSync(process.env.CERTIFICATE_PATH + "/server.key", "utf8") : null;
var certificate = PRODUCTION ? _fs2.default.readFileSync(process.env.CERTIFICATE_PATH + "/server.cer", "utf8") : null;

var credentials = PRODUCTION ? { key: privateKey, cert: certificate } : null;
var config = {
  user: process.env.PRIMAVERA_USER,
  password: process.env.PRIMAVERA_PASS,
  encrypt: true,
  server: process.env.PRIMAVERA_SERVER + "\\" + process.env.PRIMAVERA_INSTANCE,
  database: process.env.PRIMAVERA_DATABASE,
  connectionTimeout: 300000,
  requestTimeout: 300000,
  pool: {
    idleTimeoutMillis: 300000,
    max: 100
  }
};

var init = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(params) {
    var _ref2, onlySetupSql, app, server, io;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _ref2 = params || {}, onlySetupSql = _ref2.onlySetupSql;

            console.debug("BEGIN API INITIALIZATION:");
            console.debug("SERVER HOSTNAME: ", global.HOSTNAME);
            console.debug("UPLOAD PATH: ", global.UPLOAD_PATH);
            console.debug("CRM PROXY: ", global.DYNAMICS);
            console.debug("ERP PROXY: ", global.ERP);
            console.debug("GRAPHQL PORT: ", global.GRAPHQL_PORT);
            console.debug("MAILER MODE: ", process.env.MAIL_DEVELOPER === "false" ? "Normal" : "Developer");

            _context.next = 10;
            return _mssql2.default.connect(config);

          case 10:
            if (!onlySetupSql) {
              _context.next = 12;
              break;
            }

            return _context.abrupt("return");

          case 12:
            console.debug("MSSQL CONNECTION ESTABLISHED");

            app = (0, _express2.default)();

            (0, _extensions.Prevention)(app);
            (0, _extensions.ServePicture)(app);
            app.use((0, _cors2.default)());

            app.use("/graphqli", function (req, res, next) {
              return (0, _expressGraphql2.default)({
                rootValue: (0, _graphql.rootValue)({ io: io, req: req, cucm: _CUCM2.default }),
                schema: _graphql.schema,
                graphiql: true,
                limit: 99999999
              })(req, res, next);
            });
            app.use("/graphql", function (req, res, next) {
              return (0, _expressGraphql2.default)({
                rootValue: (0, _graphql.rootValue)({ io: io, req: req, cucm: _CUCM2.default }),
                schema: _graphql.schema,
                graphiql: false,
                limit: 99999999
              })(req, res, next);
            });
            console.debug("GRAPHQL API INITIALIZED on port " + global.GRAPHQL_PORT);
            _context.next = 22;
            return (0, _generateFlags2.default)();

          case 22:

            app.use((0, _compression2.default)({
              level: 7,
              filter: function filter() {
                return true;
              }
            }));

            server = credentials ? _https2.default.createServer(credentials, app) : _http2.default.createServer(app);

            server.listen(global.GRAPHQL_PORT);

            _context.next = 27;
            return (0, _io2.default)(server);

          case 27:
            io = _context.sent;

            (0, _scheduledTasks2.default)();

          case 29:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function init(_x) {
    return _ref.apply(this, arguments);
  };
}();

!process.env.DRYRUN && init();

exports.default = init;