"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _ = require("../");

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var generateFetchQuery = function generateFetchQuery(_ref) {
  var name = _ref.name,
      includes = _ref.includes;

  return "SELECT " + name + ".*" + (includes ? selectFromJoins(includes) : "") + "\n  FROM " + name + (includes ? resolveJoins({ includes: includes, name: name }) : "");
};

var groupIncludes = function groupIncludes(_ref2) {
  var table = _ref2.table,
      tableAlias = _ref2.tableAlias,
      fields = _ref2.fields;
  return fields.map(function (_ref3) {
    var field = _ref3.field,
        aggregator = _ref3.aggregator,
        dontGroup = _ref3.dontGroup;
    return aggregator || dontGroup ? null : (tableAlias || table) + "." + field;
  }).filter(function (field) {
    return field;
  }).join(", ");
};

var resolveGrouping = function resolveGrouping(_ref4) {
  var name = _ref4.name,
      fields = _ref4.fields,
      includes = _ref4.includes;

  var groupedIncludes = includes ? includes.map(groupIncludes).filter(function (entry) {
    return entry;
  }).join(", ") : null;
  return " GROUP BY " + fieldNames(name, fields).join(", ") + (groupedIncludes ? ", " + groupedIncludes : "");
};

var fieldNames = function fieldNames(name, fields) {
  var res = [];
  for (var fieldName in fields) {
    if (fields.hasOwnProperty(fieldName)) {
      !fields[fieldName].proxy && res.push(name + "." + fieldName);
    }
  }
  return res;
};

var selectFromJoins = function selectFromJoins(includes) {
  return ", " + includes.map(function (_ref5) {
    var table = _ref5.table,
        tableAlias = _ref5.tableAlias,
        fields = _ref5.fields;
    return fields.map(function (_ref6) {
      var field = _ref6.field,
          aggregator = _ref6.aggregator,
          alias = _ref6.alias;
      return "" + (aggregator ? aggregator + "(" : "") + (tableAlias || table) + "." + field + (aggregator ? ")" : "") + (alias ? " as " + alias : "");
    }).join(", ");
  }).join(", ");
};

var resolveJoins = function resolveJoins(_ref7) {
  var includes = _ref7.includes,
      name = _ref7.name;

  var resolver = resolveJoin(name);
  return includes.map(resolver).join("\r\n");
};
var resolveJoin = function resolveJoin(name) {
  return function (_ref8) {
    var table = _ref8.table,
        through = _ref8.through,
        using = _ref8.using,
        type = _ref8.type,
        joinField = _ref8.joinField,
        tableAlias = _ref8.tableAlias;
    return " " + (type || "INNER") + " JOIN " + table + (tableAlias ? " AS " + tableAlias : "") + " ON " + (tableAlias || table) + "." + (joinField || "id") + " = " + (using || name) + "." + through;
  };
};

// INNER JOIN $TABLE AS $TABLEALIAS ON $TABLE.$JOINFIELD = $USING|$NAME.$THROUGH

var resolveFilter = function resolveFilter(filters, name) {
  var extra = 0;
  return " WHERE" + filters.map(function (filter, index) {
    var filterName = typeof filter === "string" ? filter : filter.name;
    var extras = [];
    var thisextra = 0;

    if (filter.extras) {
      for (var i = 1; i <= filter.extras.length; i++) {
        extras.push("$" + (i + index + extra));
        thisextra++;
      }
      extra += thisextra - 1;
    }

    return " " + (filter.table || name) + "." + filterName + " " + (typeof filter === "string" ? "=" : filter.type || "=") + " " + (extras.length ? "( " + extras.join(",") + " )" : "$" + (extra + index + 1));
  }).join(" AND ");
};

exports.default = function (_config) {
  return function () {
    var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(filter, params, limit) {
      var config, initialQuery, sort, order, query, result;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              config = _extends({}, _config, { fields: typeof _config.fields === "function" ? _config.fields() : _config.fields });
              initialQuery = generateFetchQuery(config, limit);
              sort = config.sort, order = config.order;
              query = initialQuery + "\n      " + (filter && filter.length ? resolveFilter(filter, config.name) : "") + "\n      " + resolveGrouping(config) + "\n      " + (sort ? "ORDER BY " + sort + " " + (order ? order : "ASC") : "") + "\n      " + (limit ? "LIMIT " + limit : "");
              _context.next = 6;
              return (0, _.psqlQuery)(query, params);

            case 6:
              result = _context.sent;
              return _context.abrupt("return", result.rows);

            case 8:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, undefined);
    }));

    return function (_x, _x2, _x3) {
      return _ref9.apply(this, arguments);
    };
  }();
};