/* eslint-env node*/
"use strict"

require( "dotenv" ).config( { path: `./${ process.env.ENV }.env` } )

var path = require( "path" )
var webpack = require( "webpack" )
var HtmlWebpackPlugin = require( "html-webpack-plugin" )


var CSSLoader =
  "css-loader?sourceMap&-minimize&modules&importLoaders=1&localIdentName=[name]__[local]__[hash:base64:5]"

module.exports = {
  resolve: {
    extensions: [ ".jsx", ".js" ],
    symlinks: false,
    cacheWithContext: false,
    modules: [
      "node_modules",
    ],
    alias: {
      app: path.resolve( __dirname, "app" ),
    },
  },
  entry: [
    "babel-polyfill",
    path.join( __dirname, "app/globals.js" ),
    path.join( __dirname, "app/index.js" ),
  ],
  output: {
    path: path.join( __dirname, "dist" ),
    filename: "[name].bundle.js",
    chunkFilename: "[name].bundle.js",
    publicPath: "/",
  },
  plugins: [
    new webpack.optimize.OccurrenceOrderPlugin(),
    new HtmlWebpackPlugin( {
      template: "app/index.tpl.html",
      inject: "body",
      filename: "index.html",
    } ),
    new webpack.NoEmitOnErrorsPlugin(),
    new webpack.DefinePlugin( {
      API_URL: JSON.stringify( process.env.API_URL ),
      __DEV__: false,
      __LIVE__: false,
      NODE_ENV: JSON.stringify( "production" ),
      "process.env.NODE_ENV": JSON.stringify( "production" ),
    } ),
  ],
  module: {
    loaders: [
      {
        test: /\.(js|jsx)?$/,
        include: path.join( __dirname, "app" ),
        exclude: /node_modules/,
        loader: "babel-loader",
        query: {
          presets: [ "react", "stage-0", "es2015" ],
          plugins: [ "babel-plugin-transform-class-properties", "transform-decorators-legacy" ],
        },
      },
      {
        test: /\.css$/,
        include: path.join( __dirname, "app" ),
        loader: "style-loader!css-loader?modules&importLoaders=1&localIdentName=[name]__[local]___[hash:base64:5]!less-loader!postcss-loader",
      },
      {
        test: /\.(jpe?g|png|gif)$/i,
        include: path.join( __dirname, "app" ),
        loaders: [
          "file-loader?hash=sha512&digest=hex&name=[hash].[ext]",
          "image-webpack-loader",
        ],
      },
      {
        test: /\.svg$/i,
        include: path.join( __dirname, "app" ),
        loaders: [
          "file-loader?hash=sha512&digest=hex&name=[hash].[ext]",
        ],
      },
      {
        test: /\.json$/,
        include: path.join( __dirname, "app" ),
        loader: "json-loader",
      },
      {
        test: /\.css$/,
        loaders: [ "style-loader", CSSLoader ],
        include: path.join( __dirname, "/node_modules/font-awesome" ),
      },
      {
        test: /\.scss$/,
        loaders: [ "style-loader", CSSLoader ],
        include: path.join( __dirname, "/app/components/NotificationsTheme/styles/" ),
      },
      {
        test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: "url-loader?limit=10000&minetype=application/font-woff",
        include: path.join( __dirname, "/node_modules/font-awesome" ),
      },
      {
        test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: "file-loader",
        include: path.join( __dirname, "/node_modules/font-awesome" ),
      },
    ],
  },
}
