#!/bin/node
import { transformFileSync } from "babel-core"

import "babel-polyfill"
import recursive from "recursive-readdir"
import path from "path"
import client from "./exception/client"

const atNodeModules = file => path.resolve( __dirname + `/../node_modules/${ file }` )

const babelOptions = {
  "presets": [ atNodeModules( "babel-preset-react" ), atNodeModules( "babel-preset-es2015" ), atNodeModules( "babel-preset-stage-0" ) ],
  "plugins": [ atNodeModules( "babel-plugin-transform-decorators-legacy" ) ],
}

const listFiles = ( path, filter = [] ) => new Promise( ( resolve, reject ) =>
  recursive( path, filter, ( error, files ) => error ? reject( error ) : resolve( files ) )
)

const server = async () => {
  const cwd = process.cwd()
  const modulesDir = `${ cwd }/modules/`
  console.debug( `__dirname: ${ __dirname }` )
  console.debug( `cwd: ${ cwd }` )
  console.debug( `modulesDir: ${ modulesDir }` )
  const files = await listFiles( modulesDir, [ "!index.js" ] )

  console.log( files )

  files.map( modulePath => {
    const relativePath = modulePath.replace( modulesDir, "" )
    console.log( relativePath )
    const result = transformFileSync( modulePath, babelOptions ).code
    console.debug( result )
  } )
}

const commands = {
  server,
  client,
}

const init = async () => {
  console.log( process.argv )

  const command = process.argv[ 2 ] || "server"

  commands[ command ]()
}

init()

export default init
