import { psqlQuery } from "../"

const parseValues = ( values, parse ) => parse.mapping.map( map => values[ map.key ] )

const generateCreatorQuery = ( name, fields, record ) => {
  const parsedKeys = parseKeys( record, fields )
  return {
    parse: parsedKeys,
    query: `INSERT INTO ${ name }( ${
      parsedKeys.keys.length > 0 ? parsedKeys.keys : "id"
    } ) VALUES ( ${ parsedKeys.values.length > 0 ? parsedKeys.values : "default" } ) RETURNING id`,
  }
}

const parseKeys = ( record, fields ) => {
  const result = { keys: [], values: [], mapping: [] }
  for ( let key in record ) {
    if ( record.hasOwnProperty( key ) && fields[ key ] !== undefined ) {
      result.keys.push( `"${ key }"` )
      result.values.push( `$${ result.values.length + 1 }` )
      result.mapping.push( { key, value: `$${ result.values.length }` } )
    }
  }
  return result
}

const insertValues = async ( fields, values, root ) => {
  let res = {}
  for ( let fieldKey in fields ) {
    if ( fields.hasOwnProperty( fieldKey ) ) {
      const field = fields[ fieldKey ]
      if ( field.insert ) {
        const insertResult = ( await field.insert( root, values[ fieldKey ], values ) )
        if ( insertResult === false ) return false
        res = {
          ...res,
          ...insertResult,
        }
      }
    }
  }
  return res
}

export default config =>
  async ( root, _values ) => {
    const { name, fields: _fields } = config
    const fields = typeof _fields === "function" ? _fields() : _fields
    const values = {}
    for ( let key in _values ) {
      if ( _values.hasOwnProperty( key ) && fields.hasOwnProperty( key ) && !fields[ key ].proxy ) {
        const value = _values[ key ]
        if ( value !== undefined ) {
          values[ key ] = value
        }
      }
    }
    const insertRecord = ( await insertValues( fields, values, root ) )
    if ( !insertRecord ) return false
    let record = {
      ...values,
      ...insertRecord,
    }
    for ( var fieldName in fields ) {
      if ( fields.hasOwnProperty( fieldName ) ) {
        const field = fields[ fieldName ]
        if ( field.dontInsert ) {
          delete record[ fieldName ]
        }
      }
    }
    const { query, parse } = generateCreatorQuery( name, fields, record )
    const result = await psqlQuery( query, parseValues( record, parse ) )
    return { id: result.rows[ 0 ].id, record: record }
  }
