import { psqlQuery } from "../"

const generateFetchQuery = ( { name, includes } ) => {
  return `SELECT ${ name }.*${ includes ? selectFromJoins( includes ) : "" }
  FROM ${ name }${ includes ? resolveJoins( { includes, name } ) : "" }`
}

const groupIncludes = ( { table, tableAlias, fields } ) =>
  fields.map( ( { field, aggregator, dontGroup } ) =>
    aggregator || dontGroup ? null : `${ tableAlias || table }.${ field }`
  ).filter( field => field ).join( ", " )

const resolveGrouping = ( { name, fields, includes } ) => {
  const groupedIncludes = includes ? includes.map( groupIncludes ).filter( entry => entry ).join( ", " ) : null
  return ` GROUP BY ${ fieldNames( name, fields ).join( ", " ) }${ groupedIncludes ? `, ${ groupedIncludes }` : "" }`
}

const fieldNames = ( name, fields ) => {
  let res = []
  for ( let fieldName in ( fields ) ) {
    if ( fields.hasOwnProperty( fieldName ) ) {
      !fields[ fieldName ].proxy && res.push( `${ name }.${ fieldName }` )
    }
  }
  return res
}

const selectFromJoins = includes =>
  `, ${ includes.map( ( { table, tableAlias, fields } ) =>
    fields.map( ( { field, aggregator, alias } ) =>
      `${ aggregator ? `${ aggregator }(` : "" }${ tableAlias || table }.${ field }${ aggregator ? ")" : "" }${ alias ? ` as ${ alias }` : "" }`
    ).join( ", " )
  ).join( ", " ) }`

const resolveJoins = ( { includes, name } ) => {
  const resolver = resolveJoin( name )
  return includes.map( resolver ).join( "\r\n" )
}
const resolveJoin = name =>
  ( { table, through, using, type, joinField, tableAlias } ) =>
    ` ${ type || "INNER" } JOIN ${ table }${ tableAlias ? ` AS ${ tableAlias }` : "" } ON ${ tableAlias || table }.${ joinField || "id" } = ${ using || name }.${ through }`

// INNER JOIN $TABLE AS $TABLEALIAS ON $TABLE.$JOINFIELD = $USING|$NAME.$THROUGH

const resolveFilter = ( filters, name ) => {
  let extra = 0
  return ` WHERE${ filters.map( ( filter, index ) => {
    const filterName = typeof filter === "string" ? filter : filter.name
    let extras = []
    let thisextra = 0

    if ( filter.extras ) {
      for ( let i = 1; i <= filter.extras.length; i++ ) {
        extras.push( `$${ i + index + extra }` )
        thisextra++
      }
      extra += ( thisextra - 1 )
    }

    return ` ${ filter.table || name }.${ filterName } ${
      typeof filter === "string" ? "=" : ( filter.type || "=" )
    } ${ extras.length ? `( ${ extras.join( "," ) } )` : `$${ extra + index + 1 }` }`
  } ).join( " AND " ) }`
}

export default _config => {
  return async ( filter, params, limit ) => {
    const config = { ..._config, fields: typeof _config.fields === "function" ? _config.fields() : _config.fields }
    const initialQuery = generateFetchQuery( config, limit )
    const { sort, order } = config
    const query = (
      `${ initialQuery }
      ${ filter && filter.length ? resolveFilter( filter, config.name ) : "" }
      ${ resolveGrouping( config ) }
      ${ sort ? `ORDER BY ${ sort } ${ order ? order : "ASC" }` : "" }
      ${ limit ? `LIMIT ${ limit }` : "" }`
    )
    const result = await psqlQuery( query, params )
    return result.rows
  }
}
