import { psqlQuery } from "../"

const generateUpdaterQuery = ( name, fields, record, selector ) => {
  const parsedKeys = parseKeys( record, fields )
  return {
    values: parsedKeys.values,
    query: `UPDATE ${ name } SET ${ parsedKeys.pairs.join( ", " ) } WHERE id=${ parseInt( selector ) }`,
  }
}

const parseKeys = ( record, fields ) => {
  const result = { pairs: [], values: [] }
  for ( let key in record ) {
    if ( record.hasOwnProperty( key ) && fields[ key ] ) {
      result.values.push( record[ key ] )
      result.pairs.push( `"${ key }" = $${ result.values.length }` )
    }
  }
  return result
}

const updateValues = async ( fields, values, root, updatee ) => {
  let res = {}
  for ( let fieldKey in fields ) {
    if ( fields.hasOwnProperty( fieldKey ) ) {
      const field = fields[ fieldKey ]
      if ( field.update && values[ fieldKey ] !== undefined ) {
        res = {
          ...res,
          ...await field.update( root, values[ fieldKey ], values, updatee ),
        }
      }
    }
  }
  return res
}

export default config =>
  async ( root, values, selector, updatee ) => {
    const { name, fields: _fields } = config
    const fields = typeof _fields === "function" ? _fields() : _fields
    const record = {
      ...values,
      ...await updateValues( fields, values, root, updatee ),
    }
    const queryConfig = generateUpdaterQuery( name, fields, record, selector )
    const result = await psqlQuery( queryConfig.query, queryConfig.values )
    return result.rowCount > 0
  }
